/*
 * Copyright (c) 1991,1993 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /usr/src/mash/repository/vint/nam-1/node.h,v 1.7 1997/11/11 19:25:48 haoboy Exp $ (LBL)
 */

#ifndef nam_node_h
#define nam_node_h

class Queue;
class Edge;
class Agent;
class Route;
class Monitor;
class NetView;

#include <math.h>
#include "animation.h"

class Node : public Animation {
    public:
	inline const char* name() const { return (label_); }
	inline int num() const { return (nn_); }
	inline double size() const { return (size_); }
	inline Agent* agents() const { return (agents_); }
	virtual void size(double s);
	virtual void reset(double);
	inline double x() const { return (x_); }
	inline double y() const { return (y_); }
	inline double dx() const { return (dx_); }
	inline double dy() const { return (dy_); }
	void displace(double dx, double dy) {
		dx_ = dx; dy_ = dy;
	}
	inline double distance(Node &n) const { 
		return sqrt((x_-n.x_)*(x_-n.x_) + (y_-n.y_)*(y_-n.y_));
	}
	inline double angle(Node &n) const {
		return atan2(n.y_-y_, n.x_-x_);
	}
	virtual void place(double x, double y);
	void label(const char* name, int anchor);
	inline int anchor() const { return (anchor_); }
	inline void anchor(int v) { anchor_ = v; }
	void add_link(Edge*);
	void add_agent(Agent*);
	void add_route(Route*);	
	void delete_route(Route*);
	void delete_agent(Agent*);
	Route *find_route(Edge *e, int group, int pktsrc, int oif) const;
	Agent *find_agent(char *name) const;
	int no_of_routes(Edge *e) const;
	int inside(double, float, float) const;
	const char* info() const;
	const char* getname() const;
	void monitor(Monitor *m, double now, char *result, int len);
	inline Edge* links() const { return (links_); }
	inline int marked() const { return (mark_); }
	inline void mark(int v) { mark_ = v; }

	void init_color(char *clr);
	void set_down(char *color);
	void set_up();
	inline int isdown() const { return (state_ == DOWN); }

	Node* next_;
    protected:
	Node(const char* name, double size);
	void update_bb();
	void drawlabel(NetView*) const;

	double size_;
	double x_, y_;
	Edge* links_;
	Route* routes_;
	Agent* agents_;
	int anchor_;
	int mark_;
	char* label_;
	int nn_;
	int state_;

	double dx_, dy_; // displacements used in AutoNetModel
};

class BoxNode : public Node {
 public:
	BoxNode(const char* name, double size);
	virtual void size(double s);
	virtual void draw(NetView*, double now) const;
	virtual void place(double x, double y);
 private:
	float x0_, y0_;
	float x1_, y1_;
};

class CircleNode : public Node {
 public:
	CircleNode(const char* name, double size);
	virtual void size(double s);
	virtual void draw(NetView*, double now) const;
 private:
	float radius_;
};

class HexagonNode : public Node {
 public:
	HexagonNode(const char* name, double size);
	virtual void size(double s);
	virtual void draw(NetView*, double now) const;
	virtual void place(double x, double y);
 private:
	float ypoly_[6];
	float xpoly_[6];
};

class VirtualNode : public Node {
 public:
	VirtualNode(const char* name);
	virtual void size(double s);
	virtual void draw(NetView*, double now) const;
	virtual void place(double x, double y);
 private:
	float x_, y_;
};

#endif
